clf
%use the twister method for number generation and set seed to 2
rand('twister',2);
M = 7; %months
Y = 5; %years

%limit the number of victories in 4 (one per week). The season has 7 months
r=floor(5*rand(M,Y));

trend = repmat([0 0 0 1 1 1 2]',1,Y); %create trend
victories = r + trend;
victories(find(victories>4))=4;

if 0 %in case the random number generator changes in future versions
    victories = [
     2     3     0     0     1
     0     1     3     2     1
     2     1     4     0     2
     3     4     3     1     2
     3     3     4     3     4
     2     1     1     2     3
     3     4     4     2     4];
end

mean(victories(:,1))
std(victories(:,1))

stem(1:M,victories(:,1)); xlabel('n (month)'); ylabel('# victories'); grid
writeEPS('randomsignal');

close all
figure1=figure %should have a better way for doing this

plot(victories);
xlabel('n (month)'); ylabel('# victories'); 
writeEPS('realizations');

% Create line
annotation(figure1,'line',[0.5179 0.5196],[0.1061 0.9167],'LineStyle','--',...
    'LineWidth',4);
% Create line
annotation(figure1,'line',[0.7754 0.7771],[0.11 0.9205],'LineStyle','--',...
    'LineWidth',4);
writeEPS('marked_realizations');

%estimate joint pdf (probability density function) through histograms
clf
[n,bin]=hist(victories(4,:));
bar(bin,n)
[n,bin]=hist(victories(6,:));
bar(bin,n/sum(n))
%joint pdf
[histogram,xaxis,yaxis]=ak_hist2d(victories(4,:),victories(6,:),7,7);
mesh(xaxis,yaxis,histogram/sum(histogram(:))); %normalize to estimate pmf
xlabel('X(4)'); ylabel('X(6)')
writeEPS('joint_pdf');

%estimate correlation function
[Rxx,n1,n2] = ak_correlation(victories);
mesh1 = mesh(n1,n2,Rxx);
ylabel('n1'); xlabel('n2'); zlabel('Rx');
%makedatatip(mesh1,[1]); %could not use makedatatip properly in this case
writeEPS('correlationNonStationary');

%generate random signal
clear all
%unipolar case
numRealizations = 1000;
numSamples = 5000;
%matrix X represents the random process
X=floor(2*rand(numSamples,numRealizations)); %random values 0 or 1
[Rxx,n1,n2] = ak_correlation(X,10); 
subplot(121); mesh2 = mesh(n1,n2,Rxx);
ylabel('n1'); xlabel('n2'); zlabel('Rx');
%makedatatip(mesh2,[10 10]); %could not use makedatatip properly in this case
axis([1 10 1 10 0 1])
title('(a) random values 0 and 1');

%convert Rxx into Rxx_tau, assuming the process is WSS
[Rxx_tau, lags] = ak_convertToWSSCorrelation(Rxx);
Rxx_tau_unipolar = Rxx_tau; %keep for later use

%polar case
X=2*[floor(2*rand(numSamples,numRealizations))-0.5]; %random values -1 or 1
[Rxx,n1,n2] = ak_correlation(X,10); 
subplot(122); mesh3 = mesh(n1,n2,Rxx);
ylabel('n1'); xlabel('n2'); zlabel('Rx');
%makedatatip(mesh3,10); %could not use makedatatip properly in this case
axis([1 10 1 10 0 1])
title('(b) random values -1 and 1');

writeEPS('correlationStationary');

[Rxx_tau, lags] = ak_convertToWSSCorrelation(Rxx);
Rxx_tau_polar = Rxx_tau; %keep for later use

clf
subplot(121); stem(lags, Rxx_tau_unipolar);
xlabel('lag k'); ylabel('Rx[k]'); title('Unipolar');
axis([-10 10 -0.2 1.2])
subplot(122); stem(lags, Rxx_tau_polar);
xlabel('lag k'); title('Polar');
axis([-10 10 -0.2 1.2])
writeEPS('correlationStationaryOneDim');

%assuming ergodicity
numSamples=1000;
Xunipolar=floor(2*rand(1,numSamples)); %random values 0 or 1
[Rxx,lags]=xcorr(Xunipolar,9,'unbiased');
subplot(121); stem(lags,Rxx)
xlabel('lag k'); ylabel('Rx[k]'); title('Unipolar');
axis([-10 10 -0.2 1.2])
Xpolar=2*[floor(2*rand(1,numSamples))-0.5]; %random values -1 or 1
[Rxx,lags]=xcorr(Xpolar,9,'unbiased');
subplot(122); stem(lags,Rxx)
xlabel('lag k'); title('Polar');
axis([-10 10 -0.2 1.2])
writeEPS('correlationErgodic');
hold off