w0=pi/4; %transmitter carrier frequency
freqOffset=pi/10; %frequency offset (to be imposed to the signal)
phaseOffset=pi/3; %phase offset (to be imposed to the signal)
M=4; % modulation order ( cardinality of the set of symbols )
N=2000; % number of symbols to be generated
L=100; % oversampling factor, needs to be large to make PAM BW small
%% 1) Transmitter processing
indices = floor(M*rand(1,N))+1; % random indices from [1, M]
alphabet = [-(M-1):2:M -1]; % symbols ..., -3, -1, 1, 3, 5, 7,...
m=alphabet(indices); % obtain N random symbols
m_upsampled = zeros(1,N*L); %pre-allocate space with zeros
m_upsampled(1:L:end)=m; % symbols with L -1 zeros in - between
rollOff=0.5; %roll-off factor for raised cosine filter
Dsym=10% group delay, at rate Rsym (input symbols rate)
p=ak_rcosine(1,L,'fir/sqrt',rollOff,Dsym); %square-root raised cosine
gdelayRC=1+(length(p)-1)/2; %group delay of this even order RC filter
s=conv(m_upsampled,p); % convolve symbols with shaping pulse
BWp=pi/L*(1+rollOff); %PAM bandwidth in baseband
n=0:length(s)-1; %discrete-time index
carrierTx=cos(w0*n); %transmitter carrier (assume phase is 0)
s=s.*carrierTx; %upconvert to carrier frequency
%% 2) Receiver processing
carrierRx=cos((w0+freqOffset)*n+phaseOffset);%Rx carrier with offsets
r=2*s.*carrierRx; %downconvert to approximately baseband
%2.1) Carrier recovery
ntaps=101; %number of filter taps, make it an odd number
freqOffsetRx = freqOffset+0.05; %Rx has to guess the freq. offset
w1=(2*freqOffsetRx-BWp)/pi; %cutoff frequencies for filter that ...
w2=(2*freqOffsetRx +BWp)/pi; %should recover sinusoid at 2*freqOffset
ff=[0 w1-0.1 w1 w2 w2+0.1 1]; %frequencies for carrier recov. filter
h=firpm(ntaps,ff,[0 0 1 1 0 0]); %design filter for carrier recovery
r_carrierRecovery=filter(h,1,r.^2); %filter the r^2 signal (not r)
R=fft(r_carrierRecovery); Nfft=length(R); %calculate FFT
[maxPeak,indexMaxPeak]=max(abs(R(1:Nfft/2))) %find the strongest peak
estimatedFrequencyOffset=(2*pi/Nfft*(indexMaxPeak-1))/2 %in rad
angle(R(indexMaxPeak)) %phase of r^2 signal
%Compensate phase added by the filter at frequency of interest:
filterPhases=angle(freqz(h,1,[0 2*estimatedFrequencyOffset pi]));
estimatedPhaseOffset=(angle(R(indexMaxPeak))-filterPhases(2))/2;%est.
phaseError = phaseOffset-estimatedPhaseOffset %check error in phase
freqError = freqOffset-estimatedFrequencyOffset %check error in freq.
%2.2) Correct carrier offsets
carrOffSet=exp(-j*(estimatedFrequencyOffset*n+estimatedPhaseOffset));
r=r.*carrOffSet; %generates complex signal with offsets subtracted
r=2*real(r); %convert to real signal and take in account factor of 2
%2.3) Demodulate
r=filter(p,1,r); %uses a matched filter (not p is symmetric and real)
startSample=2*gdelayRC; %take in account the two filtering processes
symbolsRx = r(startSample:L:startSample+(L-1)*N); %extract symbols
