defineInputLength=12; %input length is 2^defineInputLength-1
SNRdB = 10; %SNR (dB) that determines noise power. Inf is noiseless
inputSequence=3 %choose between following options
switch inputSequence %obs: input signal must be a column vector
    case 1; x=transpose(1:2^defineInputLength-1); %arbitrary input
    case 2; x=2*((rand(2^defineInputLength-1,1)>0.5)-0.5); %random
    case 3; x=mseq(2,defineInputLength); %"maximum length" sequence
end
length(x)
%% Define the channel. Example: a Chebyshev filter
N=6; R=0.5; Wp=[0.3 0.5]; %order, ripple in dB and cutoff frequencies
[B,A]=cheby1(N,R,Wp); %design filter, obs: can check with freqz(B,A)
h=impz(B,A); %get long h (length determined by impzlength.m)
h_power = abs(h).^2; %instantaneous power of h (assumed in Watts)
h_totalEnergy=sum(h_power); %total energy of h (assumed in Joules)
accumulatedEnergySum = cumsum(h_power); %accumulated sum
energyThreshold = 99.99/100; %keep 99.99% of total energy in h
index=find(accumulatedEnergySum > energyThreshold*h_totalEnergy,1);
h=h(1:index); %truncated impulse response, can check with freqz(h)
y=conv(x,h); %pass input signal through the channel
if ~isinf(SNRdB) %add noise if requested
    Power_y = mean(abs(y).^2);
    Power_noise = Power_y / 10^(0.1*SNRdB);
    y=y + sqrt(Power_noise)*randn(size(y)); %add some noise
end
X=convmtx(x,length(h)); %prepare for convolution as matrix
%% Compare identification techniques:
%1) LS estimate via Moore-Penrose pseudoinverse
h_hat = pinv(X)*y;  %pinv is a robust estimation of the pseudoinverse
mse_pinv = mean((h-h_hat).^2) %estimation error
%2) Assuming inv(X'*X) is alpha*I  (where I is the identity matrix)
h_hat2 = X'*y; %under assumption, no need to calculate pinv(X)
alpha = 1/sum(x.^2); %find normalization factor
z=conv(y,flipud(x))*alpha; %z is equal to z2
%z=conv(y,x)*alpha; %z is equal to z2
[maxOutput, maxIndex] = max(abs(z)); %find peak of output (R)
M=length(h)-1;
delay=18; %Why?????
h_hat3=z(maxIndex-delay:maxIndex-delay+M) %estimated channel

%alpha=sqrt(mean(abs(h).^2)/mean(abs(h_hat2).^2)); %ideal value
h_hat2 = h_hat2 * alpha; %normalize
mse_simplified3 = mean((h-h_hat3).^2) %error in the estimation
mse_simplified = mean((h-h_hat2).^2) %error in the estimation
SNRye_pinv = 10*log10(mean(abs(y).^2)/mse_pinv)
SNRye_simplified = 10*log10(mean(abs(y).^2)/mse_simplified)
SNRye_simplified3 = 10*log10(mean(abs(y).^2)/mse_simplified3)