function [Rxx,n1,n2]=ak_correlation(X, maxTime)
% function [Rxx,n1,n2]=ak_correlation(X, maxTime)
%Estimate correlation for non-stationary random processes.
%inputs: X, matrix representing discrete-time process
% maxTime, maximum value of lags. Default: number of rows 
%outputs: Rxx[n1,n2]=E[X[n1] X*[n2]]
% n1,n2, lags to properly plot the axes
%Each column is a realization of X.
%The values of n1,n2 are varied from 1 to maxTime.
[numSamples, numRealizations] = size(X);
if nargin==1
    maxTime = numSamples; %assume maximum possible value
end
Rxx = zeros(maxTime,maxTime); %pre-allocate space
for n1=1:maxTime
    for n2=1:maxTime
        Rxx(n1,n2)=mean(X(n1,:).*conj(X(n2,:))); %estimate
    end
end
if nargout < 1 %plot if there are not output arguments
  mesh(n1,n2,Rxx);ylabel('n1');xlabel('n2');zlabel('Rxx');
else
  n1=1:maxTime; n2=1:maxTime; %create output axes
end
