function [region, x1lim, x2lim, delta1, delta2] = getregion(x1, x2, x3, N_rateregion)
% function region = getregion(x1, x2, x3, N_rateregion)
% Creates a grid of N_rateregion x N_rateregion points
% with x2 as abscissa and x1 as ordinate. x3 is the z-dimension, such that
% region(x2,x1)=x3.

x1max = max(x1);
x2max = max(x2);

delta1 = x1max/(N_rateregion-1);
delta2 = x2max/(N_rateregion-1);

region=zeros(N_rateregion,N_rateregion);

%note that for mesh(x,y,z), x are the columns and y the rows. Example:
%x=rand(2,10)
%mesh(1:10,1:2,x)

for i=1:length(x1)
    ndx1 = 1+round(x1(i)/delta1);
    ndx2 = 1+round(x2(i)/delta2);
    
    if (region(ndx1,ndx2) == 0)    
        %never used, update value
        region(ndx1,ndx2) = x3(i);
    else
        %already used
        if (x3(i) < region(ndx1,ndx2))
            region(ndx1,ndx2) = x3(i);
        end
    end
end

x2lim=0:delta2:x2max;
x1lim=0:delta1:x1max;
