%Using only K=2 tones, show how the rate varies with the
%power allocation. Could also show for K=3 assuming that
%the modem would use its total power, such that:
%P3 = Pmax - P1 - P2
clear all
close all
format long %show numbers as 15-digit scaled
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definitions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if 0 %pick channel 1 or 2
    %G{1,1} = [1e-3 1e-6];  %CO-long line: bad direct channel, better is tone 1
    %Gk1 = 1e-10;
    %Gk2 = 1e-9;
    Gk1 = 1e-3;
    Gk2 = 1e-6;
else
    %G{2,2} = [1 1e-2]; %RT-short line: good direct channel
    Gk1 = 1;
    Gk2 = 1e-6;
end
%ADSL has maximum power = 110 mW, which gives 110m / 256 ~= 4.3e-4 W per
%tone in this case. Because this simulation adopts K=2 tones, I will
%assume the total power to be 2*4.3e-4=8.6e-4~=1 mW, which is 0 dBm.
deltaf = 4.13125e3; %frequency spacing
symbolRate = 4000; %fs = 4 kHz
%p_max = 1e-3 / deltaf; %max(delta_s);  %maximum power per user. it should be 1 mW
%p_max = 2e-7; %maximum power in Watts
p_max = 1e-3; %maximum power in Watts

%Assume -140 dBm/Hz as background noise
%1e-17 Watts = 10*log10(1e-17/1e-3) = -140 dBm
noise_power = 1e-17 * deltaf; %should be / instead of * ?
n_power_levels  = 1000;

%adopt the maximum level as the total power such that one tone can get all
%the total power
max_power_dBmHz = 10*log10(p_max/1e-3);
min_power_dBmHz = -50;

%reserve one level to 0 Watts, so use n_power_levels-1
if 1
    %use linear spacing in dB domain
    delta_s = linspace(min_power_dBmHz,max_power_dBmHz,n_power_levels-2);
    %convert to Watts: dBm = 10 log10 (W/1e-3) and pre-append 0 Watts and
    %also include half of p_max
    delta_s = [0 p_max/2 (1e-3 * (10.^ (delta_s/10)))];
    delta_s = sort(delta_s); %because of the inclusion of p_max/2
    %correct numerical errors
    delta_s(end) = p_max;
else
    %use linear spacing in Watts domain
    delta_s = linspace(3.1623e-006,0,n_power_levels-1);    
    delta_s = [0 delta_s];
end

S1=zeros(1,2);
best = -1e300;

[s1,s2] = meshgrid(delta_s, p_max-delta_s);
s2=s2'; %Matlab requires me to do that, such that s1 + s2 = p_max

%assume both tones can transmit QAM (no need for PAM)
bitsk1 = log2(1 + (s1*Gk1)/noise_power);
bitsk2 = log2(1 + (s2*Gk2)/noise_power);

rate = symbolRate * (bitsk1 + bitsk2);

meshc(s1,s2,rate);
xlabel('Power for tone k=1')
ylabel('Power for tone k=2')
%colorbar
axis tight; hold on

%find maximum
maxRate = max(rate(:));
maxndx = find (rate == maxRate);
bests1 = s1(maxndx);
bests2 = s2(maxndx);

sk1 = bests1(1)
sk2 = bests2(1)
maxRate

bitsk1 = log2(1 + (sk1*Gk1)/noise_power)
bitsk2 = log2(1 + (sk2*Gk2)/noise_power)
symbolRate * (bitsk1 + bitsk2)

handler = plot3(bests1,bests2,maxRate*ones(size(maxndx)),'rh','markersize',16)
makedatatip(handler,[maxndx(1),maxndx(1)])
%figure(1); print -depsc oneuser.eps
