%Gabarito questao 1 da prova
clear all, close all
showPlots = 1;

N=6 %FFT number of points
h=10^(-6)*[1 1 0 0 1]; %channel

L = 4; %length of cyclic prefix = channel dispersion
disp('The length of the cyclic prefix should be at least 4');

totalPower_dBm = 40;
totalPower = 1e-3 * 10^(totalPower_dBm/10) %convert from dBm to Watts

noisePSD_dBm = -110;
noisePSD_N0div2 = 1e-3 * 10^(noisePSD_dBm/10) %convert from dBm to Watts

margin_dB = 3 %margin in dB
margin = 10^(0.1*margin_dB) %convert to linear scale

Pe = 1e-3; %target error probability
gap = 1/3 * qfuncinv(Pe/4)^2 %gap calculation
gap_dB = 10 * log10(gap) %convert linear to dB

%find DMT symbol rate in Hz (or bauds) from sampling frequency
fs = 100e3; %sampling frequency in Hz
Ts = 1/fs;
Tdmt = (N+L)*Ts;
%item a) 
Rdmt = 1/Tdmt %bauds

%noise per subchannel
twiceSigma2 = 2 * noisePSD_N0div2;

if showPlots == 1
    freqz(h);
    title('Channel frequency response (positive frequencies only)')
    pause
end
Hk = fft(h,N); %use FFT of length N to get channel frequency response
g = abs(Hk).^2

%turn off the PAM subchannels assigning 0 gain to them
g(1)=0;
g(N/2+1)=0;

%now do waterfilling: all subchannels are submitted to the same noise
[txPower,waterlevel,SNR]= ...
    ak_simplewaterfill(g,twiceSigma2,gap,totalPower);

if showPlots == 1
    %plot waterfilling figure in linear scale
    A=[gap * (twiceSigma2./g);
        txPower];
    bar(A','stacked')
    xlabel('Tones (k)')
    ylabel('Stacked: inverse SNR (\Gamma\sigma/g) and Tx power')
    hold on
    plot(1:length(txPower),waterlevel*ones(size(txPower)),'-X','markersize',16);
    hold off
    pause
end

%sanity check
if (sum(txPower) - totalPower > 1e-15) 
    error('sum(txPower) differs from totalPower!')
end

disp('b) SNR not showing the negative subchannels:')
%b)
SNR = SNR(1:N/2+1)
SNRdB = 10*log10(SNR)

disp('c) non-integer # bits not showing the negative subchannels:')
bitsPerDimension=0.5*log2(1 + SNR/(gap*margin));
bits = 2*bitsPerDimension %valid for QAM subchannels

%margin calculation (Eq. 4.7 in page 296 of Cioffi's notes):
disp('d) margin assuming a non-integer bitloading');
lambda = (SNR/gap) ./ (2.^(2*bitsPerDimension) - 1)
lambda_dB = 10*log10(lambda)

disp('e) Assuming an integer (floored) bitloading');
bits = floor(bits)
bitsPerDimension = bits/2;
disp('f) New margin assuming the integer bitloading');
lambda = (SNR/gap) ./ (2.^(2*bitsPerDimension) - 1)
lambda_dB = 10*log10(lambda)

disp('g) Constellations assuming the integer bitloading');
%1) Design Gray-coded constellations for 4 and 5 bits per symbol
M = 2^4;
x = [0:M-1];
consM16 = qammod(x,M,[],'gray'); %QAM

M = 2^5;
x = [0:M-1];
consM32 = qammod(x,M,[],'gray'); %QAM

%2) Normalize to have power equal to one
consM16 = consM16 / sqrt(mean(abs(consM16).^2));
consM32 = consM32 / sqrt(mean(abs(consM32).^2));

if showPlots == 1
    plotConstellation(16, consM16); pause   
    plotConstellation(32, consM32); pause    
end

%3) Find new scale factors and constellations
%first get result from waterfilling for non-negative frequencies:
targetPower = txPower(1:N/2+1);
disp('h) Scale factors for constellations of positive tones:')
scaleFactors = sqrt(targetPower)

disp('i) FEQ:')
Gk = 1 ./ Hk %Equalizador FEQ

disp('j) delta_f')
delta_f = fs / N

disp('k) P_e for subchannels:')
M = 2 .^ bits;
PeQAMs = 4*(1-1./sqrt(M)).*qfunc(sqrt( (3./(M - 1)) .* SNR))

disp('l) Find QAM symbols when assuming 9 bits equal to 1:')
%do not forget to scale their energy using the scaling factors
X0 = 0;
X1 = scaleFactors(1) * genqammod(bin2dec('1111'),consM16);
X2 = scaleFactors(2) * genqammod(bin2dec('11111'),consM32);
X3 = 0;
X4 = conj(X2);
X5 = conj(X1);
disp('DMT symbol to be modulated and transmitted:')
X=[X0 X1 X2 X3 X4 X5]

%transmit over the channel
[A, Ai]=ak_fftmtx(N,1); %get normalized DFT matrices
X=transpose(X) %make X a column vector
x=Ai * X  %modulate
xp=[x(N-L+1:N); x]  %add CP
r=conv(xp,h)
y=r(L+1:L+N)    %take CP out and keep the N important samples
disp('m) Demodulator output Xhat:')
Xhat = A * y %demodulate
%sanity check (not required in exam):
Xest = Xhat .* transpose(Gk);  %estimacao
disp('The numerical error between X and Xhat is around 1e-15:')
sum(abs(Xest-X))

%rate
rate = sum(bits)*Rdmt