%Script qpsk_simple.m
N = 2000000; %number of symbols

%QPSK constellation
konst_qpsk = exp(j*[pi/4 3*pi/4 5*pi/4 7*pi/4]).'; % QPSK alphabet

%trick:
%define:
%x =[10    20    30    40]
%then use the indices: x([2 4 1 1 1 1])
%ans = 20    40    10    10    10    10
%it's used below:
qpsk = konst_qpsk(randint(N,1,4)+1); % QPSK symbol sequence

plot(qpsk,'o'); xlabel('Re');ylabel('Im'); % plot the constellation
axis([-2 2 -2 2]); % Skale the axis for the figure
pause

 n = randn(size(qpsk))+j*randn(size(qpsk)); % randn generates Gaussian distributed noise
% Then, we define the variances:
 svqpsk = std(qpsk)^2; % QPSK signal variance (power)

 nv = std(n)^2; % noise variance (power)
% SNR is defined as SNR = 10*log10(sv/nv) = 20*log10(std(qam/qpsk)/std(n)). SNR can be
% changed by multiplying either signal or noise vector by a proper constant p. Let us multiply the
% noise-vector by variable p => SNR = 20*log10(std(qam/qpsk)/std(p*n)). Now, p can be defined as
 SNR = 1; % SNR-ratio = 20 dB
 p1 = std(qpsk)/(std(n)*10^(SNR/20)); % constant p
 snqpsk = qpsk+n*p1; % add noise to signal
 plot(snqpsk,'o'); % plot constellation
 pause

 %Detection of QPSK
 sn_block = repmat(snqpsk,1,4).'; % 4x20000-matrix, each line contains the same sn-vector
%Note! .' is the non-conjugate (ordinary) transpose while ' is the conjugate transpose. Using a real valued vector,
%it does not matter which one is used. But, in the case of a complex valued vector, using ' changes the sign of
%imaginary part of signal which is not wanted (usually).

konst_block = repmat(konst_qpsk,1,N); % 4x20000-matrix, where each column contains the const.
 distance = abs(sn_block - konst_block); % 4x20000-matrix, whose every column contains
% the received symbol distances to all possible symbol
% constellation points
 
[y,ind_1] = min(distance); % returns the minimum distance y and the corresponding
% constellation index ind_1. Both vectors have the size of 1x20000
 qpsk_det = konst_qpsk(ind_1); % using vector ind_1, we can determine the detected symbol vector

%In QPSK, theoretical symbol error probability P can be defined as (see
%Chapter 5.5 of Barry book)
% P = 2*Q(d/2sigma) - Q(d/2sigma)^2, where Q(.) is Q-function, d the minimum distance of two
%constellation points, and sigma ^ 2 is the variance of noise real of
%imaginary part.
%There is no Q-function directly in Matlab, but it can be determined by using erfc-function (see help erfc)
%Q(x) = 0.5*erfc(x/sqrt(2))
%Here is the theoretical SEP:
 d = sqrt(2); % minimum distance d
 sigma = std(real(n*p1)); % sigma is the deviation of noise real or imaginary part
 Q = 0.5*erfc(d/(sqrt(2)*2*sigma));
 sep_theo = 2*Q - Q^2 % returns the symbol error probability
%In computer simulation, we compare transmitted symbols with detected symbols and count how many errors
%occur. To obtain the symbol error probability the number of errors is divided by the total number of symbols.
%The lower probabilities we want to consider the more symbols are needed.
 number_of_errors = sum(qpsk ~= qpsk_det); % The comparison returns value 0 when its false and value 1
 sep_simu = number_of_errors/N % when it is true.
